! Template file for a post-processing code. For performance reasons,
! we store the whole trajectory in RAM and then analyze it.
!
MODULE kernels

  IMPLICIT NONE

  INTEGER, PARAMETER :: NDIM = 3
  INTEGER, PARAMETER :: DP = SELECTED_REAL_KIND(12)

CONTAINS

  !! Read positions from a configuration file in xyz format
  SUBROUTINE read(unit,pos,box)
    INTEGER, INTENT(in)   :: unit
    REAL(DP), ALLOCATABLE :: pos(:,:,:),box(:)
    CHARACTER(6)          :: string,tmp
    INTEGER               :: i,j,npart,lines,nconf
    ! Determine number of configurations assuming
    ! the number of particles is constant
    READ(unit,*) npart
    lines = 1
    DO 
       READ(unit,*,END=10) 
       lines = lines + 1
    END DO
10  CONTINUE
    nconf = lines / (npart+2)
    ! Now store box and all positions in memory
    IF (ALLOCATED(box)) DEALLOCATE(box)
    IF (ALLOCATED(pos)) DEALLOCATE(pos)
    ALLOCATE(box(NDIM),pos(NDIM,npart,nconf))
    REWIND(unit)
    DO i = 1,nconf
       READ(unit,*) npart
       READ(unit,*) box
       DO j = 1,npart
          READ(unit,*) string,pos(:,j,i)
       END DO
    END DO
    WRITE(*,*) "Read configurations", nconf
  END SUBROUTINE read

END MODULE kernels

PROGRAM main

  USE kernels

  IMPLICIT NONE

  REAL(DP), ALLOCATABLE :: pos(:,:,:),box(:)
  INTEGER               :: unit_inp=100
  CHARACTER(256)        :: file_inp="input.xyz"

  IF (COMMAND_ARGUMENT_COUNT() /= 1) THEN
     WRITE(*,"(a)") "Usage: ./a.out <file>"
     WRITE(*,"(a)") "where <file> is the trajectory file to analyze"
     STOP 
  END IF
  CALL GET_COMMAND_ARGUMENT(1,value=file_inp)

  OPEN(unit=unit_inp,file=file_inp,status="old")
  CALL read(unit_inp,pos,box)

  !... trajectory analysis here ...

  CLOSE(unit_inp)

END PROGRAM main
